@@ -3,7 +3,6 @@
 from decimal import Decimal
 import operator
 from trytond.pool import Pool, PoolMeta
-from trytond.transaction import Transaction
 
 __all__ = ['InvoiceLine']
 __metaclass__ = PoolMeta
@@ -39,7 +38,7 @@ class InvoiceLine:
         result.append(move_line)
         move_line = move_line.copy()
         move_line['debit'], move_line['credit'] = \
-                move_line['credit'], move_line['debit']
+            move_line['credit'], move_line['debit']
         if type_.endswith('supplier'):
             move_line['account'] = self.account.id
         else:
@@ -48,7 +47,10 @@ class InvoiceLine:
         return result
 
     def get_move_line(self):
-        Move = Pool().get('stock.move')
+        pool = Pool()
+        Move = pool.get('stock.move')
+        PurchaseLine = pool.get('purchase.line')
+        SaleLine = pool.get('sale.line')
 
         result = super(InvoiceLine, self).get_move_line()
 
@@ -61,16 +63,9 @@ class InvoiceLine:
 
         moves = []
         # other types will get current cost price
-        if self.invoice.type == 'in_invoice':
-            moves = [move for purchase_line in self.purchase_lines
-                    for move in purchase_line.moves
-                    if move.state == 'done']
-        elif self.invoice.type == 'out_invoice':
-            with Transaction().set_user(0, set_context=True):
-                sale_lines = self.__class__(self.id).sale_lines
-            moves = [move for sale_line in sale_lines
-                    for move in sale_line.moves
-                    if move.state == 'done']
+        if isinstance(self.origin, (PurchaseLine, SaleLine)):
+            moves = [move for move in self.origin.moves
+                if move.state == 'done']
         if self.invoice.type == 'in_invoice':
             type_ = 'in_supplier'
         elif self.invoice.type == 'out_invoice':
