@@ -4,6 +4,7 @@ from decimal import Decimal
 from trytond.model import fields
 from trytond.pyson import Eval
 from trytond.pool import Pool, PoolMeta
+from trytond.transaction import Transaction
 
 __all__ = ['Move']
 __metaclass__ = PoolMeta
@@ -20,7 +21,7 @@ class Move:
         cls._sql_constraints += [
             ('check_anglo_saxon_quantity',
                 'CHECK(quantity >= anglo_saxon_quantity)',
-                'Anglo-Saxon quantity can not be greater than quantity!'),
+                'Anglo-Saxon quantity can not be greater than quantity.'),
             ]
 
     @staticmethod
@@ -31,13 +32,17 @@ class Move:
         pool = Pool()
         Uom = pool.get('product.uom')
         AccountMoveLine = pool.get('account.move.line')
+        Currency = pool.get('currency.currency')
         lines = super(Move, self)._get_account_stock_move_lines(type_)
         if (type_.endswith('supplier')
                 and self.product.cost_price_method == 'fixed'):
             cost_price = Uom.compute_price(self.product.default_uom,
                 self.cost_price, self.uom)
+            with Transaction().set_context(date=self.effective_date):
+                unit_price = Currency.compute(self.currency, self.unit_price,
+                    self.company.currency, round=False)
             amount = self.company.currency.round(
-                Decimal(str(self.quantity)) * (self.unit_price - cost_price))
+                Decimal(str(self.quantity)) * (unit_price - cost_price))
             if self.company.currency.is_zero(amount):
                 return lines
             account = self.product.account_stock_supplier_used
@@ -65,13 +70,15 @@ class Move:
         return lines
 
     @classmethod
-    def _get_anglo_saxon_move(cls, moves, quantity):
+    def _get_anglo_saxon_move(cls, moves, quantity, type_):
         '''
-        Generator of (move, qty) where move is the move to be consumed and qty
-        is the quantity (in the product default uom) to be consumed on this
-        move.
+        Generator of (move, qty, cost_price) where move is the move to be
+        consumed, qty is the quantity (in the product default uom) to be
+        consumed on this move and cost_price is in the company currency.
         '''
-        Uom = Pool().get('product.uom')
+        pool = Pool()
+        Uom = pool.get('product.uom')
+        Currency = pool.get('currency.currency')
 
         consumed_qty = 0.0
         for move in moves:
@@ -84,7 +91,17 @@ class Move:
                 qty = quantity - consumed_qty
             if consumed_qty >= quantity:
                 break
-            yield (move, qty)
+
+            if type_.startswith('in_'):
+                with Transaction().set_context(date=move.effective_date):
+                    unit_price = Currency.compute(move.currency,
+                        move.unit_price, move.company.currency, round=False)
+                cost_price = Uom.compute_price(move.uom,
+                        unit_price, move.product.default_uom)
+            else:
+                cost_price = move.cost_price
+
+            yield (move, qty, cost_price)
 
     @classmethod
     def update_anglo_saxon_quantity_product_cost(cls, product, moves,
@@ -93,7 +110,8 @@ class Move:
         Return the cost for quantity based on lines.
         Update anglo_saxon_quantity on the concerned moves.
         '''
-        Uom = Pool().get('product.uom')
+        pool = Pool()
+        Uom = pool.get('product.uom')
 
         for move in moves:
             assert move.product == product, 'wrong product'
@@ -105,20 +123,17 @@ class Move:
 
         cost = Decimal('0.0')
         consumed_qty = 0.0
-        for move, move_qty in cls._get_anglo_saxon_move(moves, total_qty):
+        for move, move_qty, move_cost_price in cls._get_anglo_saxon_move(
+                moves, total_qty, type_):
             consumed_qty += move_qty
 
-            if type_.startswith('in_'):
-                move_cost_price = Uom.compute_price(move.uom,
-                        move.unit_price, move.product.default_uom)
-            else:
-                move_cost_price = move.cost_price
             cost += move_cost_price * Decimal(str(move_qty))
 
-            cls.write([move], {
-                'anglo_saxon_quantity': ((move.anglo_saxon_quantity or 0.0)
-                    + move_qty),
-                })
+            with Transaction().set_user(0, set_context=True):
+                cls.write([move], {
+                    'anglo_saxon_quantity': ((move.anglo_saxon_quantity or 0.0)
+                        + move_qty),
+                    })
 
         if consumed_qty < total_qty:
             qty = total_qty - consumed_qty
